"""This module is part of the PygLibsX library.
It is licensed under an open-source, make-shift license by Matthew Roe.
That will change when it is more complete and I know what I want ot do with it."""

import math
import math3d

def calculate_mass(geom):
    if geom.otype == "Vertex":
        return 0.1
    if geom.otype == "Sphere":
        return geom.radius ** 2
    if geom.otype == "Cube":
        return geom.width * 2 + geom.height * 2 + geom.depth * 2
    return 0.1

class PGeom(object):
    def __init__(self, geom, friction=0.0,
                 mass="default"):

        self.geom = geom

        self.friction = math3d.Velocity(friction, friction, friction)

        if mass == "default":
            self.mass = calculate_mass(self.geom)
        else:
            self.mass = mass

        if self.mass == 0.0:
            self.mass = 0.1

        self.velocity = math3d.Vector(0, 0, 0)
        self.inertia = self.calculate_inertia()

        self.collisions = []
        self.already_collided_with = []

        self.was_hit = False

    def calculate_inertia(self):
        return math3d.Vector(self.mass, self.mass, self.mass) * self.velocity

    def get_velocity_from_inertia(self):
        x = self.inertia.x / self.mass
        y = self.inertia.y / self.mass
        z = self.inertia.z / self.mass
        return math3d.Vector(x, y, z)

    def push(self, x, y, z):
        self.velocity.add(math3d.Vector(x, y, z))
        self.inertia = self.calculate_inertia()
        return None

    def move(self):
        if self.geom.otype == "Vector":
            self.geom.add(self.velocity)
        else:
            self.geom.pos.add(self.velocity)
        self.velocity.sub(self.friction)
        return None

    def react_to_collisions(self):
        for i in self.collisions:
            if i in self.already_collided_with:
                continue
            x = 0
            if i.geom.pos.x < self.geom.pos.x: x = 1
            elif i.geom.pos.x > self.geom.pos.x: x = -1
            y = 0
            if i.geom.pos.y < self.geom.pos.y: y = 1
            elif i.geom.pos.y > self.geom.pos.y: y = -1
            z = 0
            if i.geom.pos.z < self.geom.pos.z: z = 1
            elif i.geom.pos.z > self.geom.pos.z: z = -1

            sa = self.inertia.length()
            ai = i.inertia.length()

            self.inertia = math3d.Vector(ai * x, ai * y, ai * z)
            i.inertia = math3d.Vector(sa * (-x), sa * (-y), sa * (-z))

            self.velocity = self.get_velocity_from_inertia()
            i.velocity = i.get_velocity_from_inertia()

            self.velocity.sub(self.friction * i.friction)
            i.velocity.sub(self.friction * i.friction)

            self.move()
            i.move()

        self.collisions = []
        self.already_collided_with = []

    def check_collision(self, other):
        a = self.geom.check_collision(other.geom)
        if a:
            self.was_hit = True
            other.was_hit = True
        return a


class World(object):
    def __init__(self, grid_size = 10):

        self.grid_size = grid_size
        self.static_objects = []
        self.dynamic_objects = []

    def step(self):
        for i in self.dynamic_objects:
            for x in self.static_objects:
                i.check_collision(x)
            for x in self.dynamic_objects:
                if not i == x:
                    i.check_collision(x)

            i.move()
