import os
import sys
import random
import math3d
import obj_loader
import core
from core import *

from buildings import *
from missile import *

_island_size = 3    # number of tiles an island is across, width by height
_tile_size = 10     # size of a tile in 3D units, width by height
_build_size = 4     # number of building tiles per terrain tile

_launch_time = 1000
_missile_damage = 15

terrain = None
objects = list()
buildings = list()

class Terrain:
    def __init__(self, mesh=None, position=(0,0,0)):
        self.mesh = mesh
        self.position = position
        self.last_launch = 0

        if self.mesh == None:
            return
        else:
            mesh.make_cgeom() # create bounding box
            self.mesh.collision_geom.pos = math3d.Vector(self.position[0], self.position[2], self.position[1])

        self.island_grid = []
        for i in range(_island_size): # construct x by x grid
            self.island_grid.append([])
            for j in range(_island_size):
                pos = (self.position[0] + (i * _tile_size), \
                       self.position[1], \
                       self.position[2] + (j * _tile_size))
                self.island_grid[i].append(TerrainTile(pos, (i, j)))

        self.building_grid = []
        # multiply by the amount of buildings that we can put on here
        for i in range(_island_size * (_build_size / 2)): # half of build size here
            self.building_grid.append([])
            for j in range(_island_size * (_build_size / 2)): # other half here
                self.building_grid.append(None)

    def think(self):
        for i in range(_island_size):
            for j in range(_island_size):
                if self.island_grid[i][j].is_alive:
                    self.island_grid[i][j].think()

        self.checkGameState() # prevent infinite targetting loop

        ticks = pygame.time.get_ticks()
        if self.last_launch + _launch_time < ticks:
            self.launchMissiles()  # launch some missiles!
            self.last_launch = ticks

    def render(self):
        for i in range(_island_size):  # render terrain tiles
            for j in range(_island_size):
                if self.island_grid[i][j].is_alive:
                    self.mesh.render((self.position[0] + (i * _tile_size), \
                                      self.position[1], \
                                      self.position[2] + (j * _tile_size)), \
                                     (0, 0, 0))

## Renders a single tile at location i, j
    def renderTile(self, i, j, color=(0.7,0.1,0.1)):
        interval = _tile_size / 2.0
        size = _tile_size / 4.0
        z_offset = 5.2

        glColor3f(*color)
        pos = self.position
        pos = (pos[0] - size, pos[1], pos[2] - size) 

        glBegin(GL_QUADS)
        # top right
        glVertex3f(pos[0] + (i * interval) + size, \
                    pos[1] + z_offset, \
                    pos[2] + (j * interval) - size)
        # top left
        glVertex3f(pos[0] + (i * interval) - size, \
                    pos[1] + z_offset, \
                    pos[2] + (j * interval) - size)
        # bottom left
        glVertex3f(pos[0] + (i * interval) - size, \
                    pos[1] + z_offset, \
                    pos[2] + (j * interval) + size)
        # bottom right
        glVertex3f(pos[0] + (i * interval) + size, \
                    pos[1] + z_offset, \
                    pos[2] + (j * interval) + size)
        glEnd()

    def renderGrid(self, color=(0.7,0.1,0.1)):
        for i in range(_island_size * (_build_size / 2)): # has to be -1 for some reason
            for j in range(_island_size * (_build_size / 2)):
                self.renderTile(i, j)

## Check object collision against each terrain tile 
    def check_collision(self, other):
        for i in range(_island_size):
            for j in range(_island_size):
                if self.island_grid[i][j].is_alive:
                    # swap y and z
                    self.mesh.collision_geom.pos = math3d.Vector( \
                                        self.position[0] + (i * _tile_size), \
                                        self.position[2] + (j * _tile_size), \
                                        self.position[1])

                    if self.mesh.collision_geom.check_collision(other.mesh.collision_geom):
                        return True  # we collided

## Deal some damage to a tile
    def takeDamage(self, tile):
        self.island_grid[tile[0]][tile[1]].hitpoints -= 50

## Launches missiles at targets, damn this is getting messy!
    def launchMissiles(self):
        global objects
        i, j = 0, 0

        while True: # target a living tile..
            i = random.randint(0,2)
            j = random.randint(0,2)
            if self.island_grid[i][j].is_alive:
                break

        target = self.island_grid[i][j]
        missile = Missile(obj_loader.OBJ(os.path.join("data", "missile.obj"), True), 
                          (100,target.position[1],target.position[2]), target)
        objects.append(missile)
        #print 'Launching missile at', i, j

    def setupTurrets(self):
        global buildings
        modeldir = 'data'

        for i in range(_island_size):
            for j in range(_island_size):
                pos = self.island_grid[i][j].position
                pos = (pos[0],pos[1]+5,pos[2])
                b = Building(obj_loader.OBJ(os.path.join(modeldir, "turret.obj"), True), pos)
                buildings.append(b)
#        test = Building(obj_loader.OBJ(os.path.join(modeldir, "turret.obj"), True), (0.0,12,1))
#        buildings.append(test)
#        test = Building(obj_loader.OBJ(os.path.join(modeldir, "turret.obj"), True), (6,10,7))
#        buildings.append(test)
#        test = Building(obj_loader.OBJ(os.path.join(modeldir, "turret.obj"), True), (3,17,10))
#        buildings.append(test)

    def checkGameState(self):
        alive = False
        for i in range(_island_size):
            for j in range(_island_size):
                if self.island_grid[i][j].is_alive:
                    alive = True

        if not alive:
            print 'You \'lose\'! Try harder next time!'
            sys.exit()

class TerrainTile:
    def __init__(self, position, index):
        self.position = position
        self.index = index
        self.hitpoints = 50
        self.is_alive = True
        self.angle = 0

    def think(self):
        if self.hitpoints <= 0:
            self.is_alive = False

    def takeDamage(self):
        self.hitpoints -= _missile_damage

class GasVent:
    def __init_(self):
        self.gas_rate = 10

    def getGas(self):
        return self.gas_rate

